import { Listener } from "discord-akairo";
import { Message } from "discord.js";
import { MessageReaction } from "discord.js";
import { User } from "discord.js";
import { Appeal } from "../../entity/Appeal";
import { decision, errorEmbed, infoEmbed, successEmbed } from "../Util";

class ReactionAdd extends Listener {
  constructor() {
    super("embedDecision", {
      emitter: "client",
      event: "messageReactionAdd",
    });
  }

  async exec(reaction: MessageReaction, user: User) {
    if (reaction.message.partial) await reaction.message.fetch();

    if (!reaction.message.guild) return;
    if (user === this.client.user) return;
    if (reaction.message.author !== this.client.user) return;
    if (!"✅❔❌".includes(reaction.emoji.name)) return;

    const embed = reaction.message.embeds[0];
    if (!embed) return;

    const footer = embed.footer.text;
    if (!footer) return;

    const id = /Appeal ID: ([A-z\d-]*)$/i.exec(footer);
    if (!id) return;

    const appeal = await Appeal.findOne(id[1]);
    if (!appeal) return;

    reaction.message.reactions.resolve(reaction.emoji.name).users.remove(user);

    switch (reaction.emoji.name) {
      case "✅":
        this.onAccept(reaction.message, appeal, user);
        break;
      case "❔":
        reaction.message.channel.send("Not implemented yet :(");
        break;
      case "❌":
        this.onReject(reaction.message, appeal, user);
        break;
    }
  }

  onAccept(message: Message, appeal: Appeal, user: User) {
    decision(
      message,
      user,
      (m) => m.content === "Yes",
      infoEmbed("Are you sure you would like to accept this appeal?")
        .setDescription(
          'Respond with "Yes" to continue.\n' +
            "Sending any other message will cancel the process."
        )
        .setFooter(appeal.id),

      () =>
        this.decisionReason(
          message,
          appeal,
          user,
          "Provide Reason",
          this.accept
        ),

      () => this.onCancel(message, appeal)
    );
  }

  onReject(message: Message, appeal: Appeal, user: User) {
    decision(
      message,
      user,
      (m) => m.content === "Yes",
      infoEmbed("Are you sure you would like to reject this appeal?")
        .setDescription(
          'Respond with "Yes" to continue.\n' +
            "Sending any other message will cancel the process."
        )
        .setFooter(appeal.id),

      () =>
        this.decisionReason(
          message,
          appeal,
          user,
          "Rejection Reason",
          this.reject
        ),

      () => this.onCancel(message, appeal)
    );
  }

  decisionReason(
    message: Message,
    appeal: Appeal,
    user: User,
    title: string,
    final: (
      message: Message,
      appeal: Appeal,
      user: User,
      reason?: string
    ) => void
  ) {
    decision(
      message,
      user,
      (m) => m.content !== "cancel",
      infoEmbed(title)
        .setDescription(
          "Enter `none` to provide no reason.\n" +
            "Enter `cancel` to cancel.\n\n" +
            "*Timeout in 120 seconds*"
        )
        .setFooter(appeal.id),
      (m) => {
        let content = m.content;
        if (content === "none") content = null;
        final(message, appeal, user, content);
      },
      () => this.onCancel(message, appeal),
      120_000
    );
  }

  async accept(message: Message, appeal: Appeal, user: User, reason?: string) {
    const banned = await message.guild
      .fetchBan(appeal.userID)
      .then(() => true)
      .catch(() => false);

    if (!banned) {
      message.channel
        .send(infoEmbed("The user is not banned. Appeal status updated."))
        .then((m) => setTimeout(() => m.delete(), 5000));

      appeal.accept(user.id, reason);
      message.reactions.removeAll();

      const embed = appeal.embed(true).setColor("#00AE86");
      message.edit(embed);
      return;
    }

    let err: Error = await message.guild.members
      .unban(appeal.userID)
      .then(() => null)
      .catch((e) => e);

    if (err) {
      message.channel
        .send(
          errorEmbed("An error occurred")
            .setDescription(
              "Could not unban user.\nThe appeal status was NOT updated."
            )
            .addField("Error", err.message)
            .setFooter(appeal.id)
        )
        .then((m) => setTimeout(() => m.delete(), 5000));
      return;
    }

    appeal.accept(user.id, reason);

    message.channel
      .send(
        successEmbed("The appeal has been accepted.")
          .setDescription(`${user.username}#${user.discriminator} is unbanned`)
          .setThumbnail(user.displayAvatarURL())
          .setFooter(appeal.id)
      )
      .then((m) => setTimeout(() => m.delete(), 5000));

    message.reactions.removeAll();
    const embed = appeal.embed(true).setColor("#00AE86");
    message.edit(embed);
  }

  reject(message: Message, appeal: Appeal, user: User, reason?: string) {
    appeal.reject(user.id, reason);

    message.channel
      .send(
        successEmbed("The appeal has been rejected.")
          .setDescription(`${appeal.userID} will stay banned.`)
          .setFooter(appeal.id)
          .addField(
            "Rejection expiry",
            appeal.rejectionExpiry?.toLocaleDateString() || "Never"
          )
      )
      .then((m) => setTimeout(() => m.delete(), 5000));

    message.reactions.removeAll();
    const embed = appeal.embed(true).setColor("#B93758");
    message.edit(embed);
  }

  onCancel(message: Message, appeal: Appeal) {
    message.channel
      .send(errorEmbed("Action cancelled.").setFooter(appeal.id))
      .then((m) => setTimeout(() => m.delete(), 5000));
  }
}

export default ReactionAdd;
